from django.apps import apps
from django.db import connection
from django.db.models.signals import pre_migrate
from pytest_factoryboy import register
from wbcore.contrib.authentication.factories import UserFactory
from wbcore.contrib.geography.tests.signals import (
    app_pre_migration as app_pre_migration_geography,
)
from wbhuman_resources.tests.signals import app_pre_migration as app_pre_migration_hr
from wbintegrator_office365.factories import (
    CallEventFactory,
    CallUserFactory,
    EventFactory,
    EventLogFactory,
    SubscriptionFactory,
    TenantUserFactory,
)

register(EventFactory)
register(EventLogFactory)
register(SubscriptionFactory)
register(TenantUserFactory)
register(CallUserFactory)
register(CallEventFactory)
register(UserFactory)

pre_migrate.connect(app_pre_migration_hr, sender=apps.get_app_config("wbintegrator_office365"))
pre_migrate.connect(app_pre_migration_geography, sender=apps.get_app_config("wbintegrator_office365"))



from rest_framework.reverse import reverse
from wbcore.metadata.configs.endpoints import EndpointViewConfig


class TenantUserEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class EventEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class CallUserEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class CallEventEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class SubscriptionEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class EventLogEndpointConfig(EndpointViewConfig):
    def get_update_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None

    def get_create_endpoint(self, **kwargs):
        return None


class EventLogEventEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return None

    def get_instance_endpoint(self, **kwargs):
        if event_id := self.view.kwargs.get("last_event_id", None):
            return reverse("wbintegrator_office365:event-eventlog-list", args=[event_id], request=self.request)
        return None



# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbintegrator_office365", "0002_remove_calendar_owner_remove_calendarevent_activity_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={
                "permissions": [("administrate_event", "Can Administrate call and calendar events")],
                "verbose_name": "Event",
                "verbose_name_plural": "Events",
            },
        ),
    ]



from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import StringPreference

general = Section("wbintegrator_office365")


@global_preferences_registry.register
class AccesTokenPreference(StringPreference):
    section = general
    name = "access_token"
    default = "0"

    verbose_name = "Access Token"
    help_text = "The access token obtained from subscriptions Microsoft for authentication"



# Generated by Django 4.1.7 on 2023-04-21 10:50

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "wbintegrator_office365",
            "0001_initial_squashed_squashed_0003_alter_calendar_owner_alter_calendarevent_organizer_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="calendar",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="activity",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="calendar",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="event",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="organizer",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="participants",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="recurrence",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="activity",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="calendar_event",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="organizer",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="participants",
        ),
        migrations.RemoveField(
            model_name="location",
            name="address",
        ),
        migrations.RemoveField(
            model_name="tenantuser",
            name="enable_synchronization_activity",
        ),
        migrations.DeleteModel(
            name="Address",
        ),
        migrations.DeleteModel(
            name="Calendar",
        ),
        migrations.DeleteModel(
            name="CalendarEvent",
        ),
        migrations.DeleteModel(
            name="CalendarEventInstance",
        ),
        migrations.DeleteModel(
            name="Location",
        ),
        migrations.DeleteModel(
            name="RecurrencePattern",
        ),
    ]



import json
import logging
from datetime import date, datetime, timedelta

import numpy as np
import pandas as pd
import plotly.graph_objects as go
from django.db import transaction
from django.db.models import Case, CharField, F, When
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from plotly.subplots import make_subplots
from rest_framework import filters
from wbcore import viewsets
from wbcore.filters import DjangoFilterBackend
from wbhuman_resources.models import EmployeeHumanResource

from wbintegrator_office365.filters import (
    CallEventFilter,
    CallEventSummaryGraphFilter,
    CallUserFilter,
    EventFilter,
    EventLogFilter,
    SubscriptionFilter,
    TenantUserFilter,
)
from wbintegrator_office365.importer import parse
from wbintegrator_office365.models import (
    CallEvent,
    CallUser,
    Event,
    EventLog,
    Subscription,
    TenantUser,
)
from wbintegrator_office365.models.event import handle_event_from_webhook
from wbintegrator_office365.serializers import (
    CallEventModelSerializer,
    CallEventRepresentationSerializer,
    CallUserModelSerializer,
    CallUserRepresentationSerializer,
    EventLogModelSerializer,
    EventLogRepresentationSerializer,
    EventModelSerializer,
    EventRepresentationSerializer,
    SubscriptionListModelSerializer,
    SubscriptionModelSerializer,
    SubscriptionRepresentationSerializer,
    TenantUserModelSerializer,
    TenantUserRepresentationSerializer,
)

from .display import (
    CallEventDisplayConfig,
    CallUserDisplayConfig,
    EventDisplayConfig,
    EventLogDisplayConfig,
    EventLogEventDisplayConfig,
    SubscriptionDisplayConfig,
    TenantUserDisplayConfig,
)
from .endpoints import (
    CallEventEndpointConfig,
    CallUserEndpointConfig,
    EventEndpointConfig,
    EventLogEndpointConfig,
    EventLogEventEndpointConfig,
    SubscriptionEndpointConfig,
    TenantUserEndpointConfig,
)
from .titles import (
    CallEventReceptionTimeTitleConfig,
    CallEventSummaryGraphTitleConfig,
    CallEventTitleConfig,
    CallUserTitleConfig,
    SubscriptionTitleConfig,
    TenantUserTitleConfig,
)

logger = logging.getLogger("wbintegrator_office365")


class TenantUserRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbintegrator_office365:tenantuserrepresentation"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    search_fields = [
        "profile__computed_str",
        "display_name",
        "mail",
        "phone",
    ]
    serializer_class = TenantUserRepresentationSerializer
    queryset = TenantUser.objects.all()


class SubscriptionRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbintegrator_office365:subscriptionrepresentation"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    ordering = ["-created"]
    search_fields = ["subscription_id"]
    serializer_class = SubscriptionRepresentationSerializer
    queryset = Subscription.objects.all()


class EventRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbintegrator_office365:eventrepresentation"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    ordering = ["-auto_inc_id"]
    search_fields = ["id"]
    serializer_class = EventRepresentationSerializer
    queryset = Event.objects.all()


class CallUserRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbintegrator_office365:calleventrepresentation"

    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    search_fields = [
        "tenant_user__tenant_id",
        "tenant_user__profile__computed_str",
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
    ]

    serializer_class = CallUserRepresentationSerializer
    queryset = CallUser.objects.all()


class CallEventRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbintegrator_office365:calleventrepresentation"
    serializer_class = CallEventRepresentationSerializer
    queryset = CallEvent.objects.all()


class TenantUserViewSet(viewsets.ModelViewSet):
    IDENTIFIER = "wbintegrator_office365:tenantuser"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    ordering_fields = [
        "id",
        "tenant_id",
        "profile",
        "display_name",
        "mail",
        "phone",
        "tenant_organization_id",
        "is_internal_organization",
    ]
    ordering = ["id"]
    search_fields = [
        "profile__computed_str",
        "display_name",
        "mail",
        "phone",
    ]

    filterset_class = TenantUserFilter

    display_config_class = TenantUserDisplayConfig
    title_config_class = TenantUserTitleConfig
    endpoint_config_class = TenantUserEndpointConfig

    serializer_class = TenantUserModelSerializer
    queryset = TenantUser.objects.all()


class SubscriptionViewSet(viewsets.ModelViewSet):
    IDENTIFIER = "wbintegrator_office365:subscription"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    ordering_fields = [
        "subscription_id",
        "change_type",
        "expiration_date",
        "type_resource",
        "is_enable",
        "tenant_user",
        "created",
    ]
    ordering = ["-created"]
    search_fields = [
        "subscription_id",
        "tenant_user__profile__computed_str",
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
    ]
    title_config_class = SubscriptionTitleConfig
    display_config_class = SubscriptionDisplayConfig
    endpoint_config_class = SubscriptionEndpointConfig

    filterset_class = SubscriptionFilter

    serializer_class = SubscriptionModelSerializer
    queryset = Subscription.objects.all()

    def get_serializer_class(self):
        if self.get_action() in ["list", "list-metadata"]:
            return SubscriptionListModelSerializer
        return super().get_serializer_class()


class CallUserViewSet(viewsets.ModelViewSet):
    IDENTIFIER = "wbintegrator_office365:calluser"
    filter_backends = (
        DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    )
    ordering_fields = ["is_guest", "is_phone", "tenant_user"]
    ordering = ["id"]
    search_fields = [
        "tenant_user__profile__computed_str",
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
    ]

    display_config_class = CallUserDisplayConfig
    title_config_class = CallUserTitleConfig
    endpoint_config_class = CallUserEndpointConfig

    filterset_class = CallUserFilter

    serializer_class = CallUserModelSerializer
    queryset = CallUser.objects.all()

    def get_queryset(self):
        qs = (
            super()
            .get_queryset()
            .annotate(
                name_user=Case(
                    When(tenant_user__profile__isnull=True, then=F("tenant_user__display_name")),
                    default=F("tenant_user__profile__computed_str"),
                    output_field=CharField(),
                ),
                phone=F("tenant_user__phone"),
                mail=F("tenant_user__mail"),
            )
        )
        return qs


class EventViewSet(viewsets.ModelViewSet):
    IDENTIFIER = "wbintegrator_office365:event"
    filter_backends = (
        DjangoFilterBackend,
        filters.OrderingFilter,
    )
    ordering_fields = [
        "auto_inc_id",
        "nb_received",
        "id",
        "type",
        "change_type",
        "tenant_user",
        "created",
        "changed",
        "subscription_id",
        "id_event",
        "is_handled",
    ]

    ordering = ["-changed", "-created"]

    filterset_class = EventFilter

    display_config_class = EventDisplayConfig
    endpoint_config_class = EventEndpointConfig

    serializer_class = EventModelSerializer

    queryset = Event.objects.all()


class EventLogRepresentationViewSet(viewsets.RepresentationViewSet):
    serializer_class = EventLogRepresentationSerializer
    queryset = EventLog.objects.all()


class EventLogViewSet(viewsets.ModelViewSet):
    filter_backends = (
        DjangoFilterBackend,
        filters.OrderingFilter,
    )
    endpoint_config_class = EventLogEndpointConfig
    display_config_class = EventLogDisplayConfig
    serializer_class = EventLogModelSerializer
    filterset_class = EventLogFilter
    ordering_fields = ["id", "order_received", "last_event", "change_type", "created", "changed", "id_event"]
    ordering = ["-changed", "-created"]

    queryset = EventLog.objects.all()


class EventLogEventViewSet(EventLogViewSet):
    endpoint_config_class = EventLogEventEndpointConfig
    display_config_class = EventLogEventDisplayConfig

    def get_queryset(self):
        return super().get_queryset().filter(last_event=self.kwargs["last_event_id"])


class CallEventViewSet(viewsets.ModelViewSet):
    IDENTIFIER = "wbintegrator_office365:callevent"
    filter_backends = (
        DjangoFilterBackend,
        filters.OrderingFilter,
    )
    ordering_fields = ["event", "organizer", "created", "start", "end", "change_type", "is_internal_call"]
    ordering = ["-created"]

    filterset_class = CallEventFilter

    display_config_class = CallEventDisplayConfig
    title_config_class = CallEventTitleConfig
    endpoint_config_class = CallEventEndpointConfig

    serializer_class = CallEventModelSerializer
    queryset = CallEvent.objects.all()

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.annotate(change_type=F("event__change_type"))


class CallEventReceptionTime(viewsets.ChartViewSet):
    IDENTIFIER = "wbintegrator_office365:calleventchart"
    queryset = CallEvent.objects.all()
    title_config_class = CallEventReceptionTimeTitleConfig

    def get_plotly(self, queryset):
        fig = go.Figure()
        if queryset.exists():
            df = pd.DataFrame(queryset.values("start", "end", "created"))
            df["delay"] = np.where(df["created"] > df["end"], df["created"] - df["end"], df["created"] - df["start"])
            df["minutes"] = df.delay.dt.seconds / 60
            df = df.loc[df["minutes"] < 1000]
            # print(df.loc[100, ["delay", "minutes"]])
            df["mean_minutes"] = df.minutes.mean()
            df["median_minutes"] = df.minutes.median()
            fig.add_trace(
                go.Scatter(
                    x=df.index,
                    y=df.minutes,
                    mode="lines+markers",
                    name="Delay",
                )
            )

            fig.add_trace(
                go.Scatter(
                    x=df.index,
                    y=df.mean_minutes,
                    mode="lines",
                    name="Average",
                )
            )

            fig.add_trace(
                go.Scatter(
                    x=df.index,
                    y=df.median_minutes,
                    mode="lines",
                    name="Median",
                )
            )

            fig.update_layout(
                title="<b>Call reception delay</b>",
                xaxis=dict(
                    title="Calls",
                ),
                yaxis=dict(
                    title="Delay in minutes",
                    # type= 'date'
                ),
                autosize=True,
                xaxis_rangeslider_visible=True,
                height=850,
            )

        return fig


class CallEventSummaryGraph(viewsets.ChartViewSet):
    filter_backends = (DjangoFilterBackend,)
    IDENTIFIER = "wbintegrator_office365:calleventsummarygraph"
    queryset = CallEvent.objects.all()
    title_config_class = CallEventSummaryGraphTitleConfig
    filterset_class = CallEventSummaryGraphFilter

    def get_queryset(self):
        return super().get_queryset().annotate(duration_seconds=F("end") - F("start"))

    def get_dataframe_business_days(self, start_date, end_date):
        temp_date = start_date
        list_date = []
        while temp_date <= end_date:
            if temp_date.weekday() <= 4:
                list_date.append(temp_date)
            temp_date += timedelta(days=1)
        return pd.DataFrame(list_date, columns=["day"])

    def merge_df_with_business_days(self, df):
        start_date = self.request.GET.get("start", None)
        end_date = self.request.GET.get("end", None)
        start_date = datetime.strptime(start_date, "%Y-%m-%d").date() if start_date and start_date != "null" else None
        end_date = datetime.strptime(end_date, "%Y-%m-%d").date() if end_date and end_date != "null" else date.today()
        end_date = end_date if end_date <= date.today() else date.today()

        if start_date and end_date:
            df_day = self.get_dataframe_business_days(start_date, end_date)
            df_day.columns = pd.MultiIndex.from_product([df_day.columns, [""]])
            df = df.merge(df_day, on=["day"], how="outer")
            df[["count", "sum", "mean", "median"]] = df[["count", "sum", "mean", "median"]].fillna(0)
            df["average_count_period"] = df["count"]["minutes"].mean()
            df["average_sum_period"] = df["sum"]["minutes"].mean()
            df["average_mean_period"] = df["mean"]["minutes"].mean()
            df["average_median_period"] = df["median"]["minutes"].mean()
            df = df.sort_values(by=["day"]).reset_index()
        return df

    def get_plotly(self, queryset):
        fig = go.Figure()
        if queryset.exists():
            df = pd.DataFrame(queryset.annotate(day=F("created__date")).values("start", "end", "day"))
            dict_df = {"employee": df}
            df_compare_employee = pd.DataFrame()
            if compare_employee_id := self.request.GET.get("compare_employee", None):
                compare_employee = EmployeeHumanResource.objects.get(id=compare_employee_id)

                qs2 = CallEvent.objects.filter(
                    participants__tenant_user__profile__computed_str__icontains=compare_employee.profile.computed_str
                )
                if start_compare := self.request.GET.get("start", None):
                    qs2 = qs2.filter(start__gte=start_compare)

                if end_compare := self.request.GET.get("end", None):
                    qs2 = qs2.filter(end__lte=end_compare)
                if call_duration := self.request.GET.get("call_duration", None):
                    qs2 = qs2.annotate(duration_seconds=F("end") - F("start")).filter(
                        duration_seconds__gte=timedelta(seconds=float(call_duration))
                    )
                if call_type := self.request.GET.get("call_type", None):
                    qs2 = qs2.filter(type=call_type)
                if call_area := self.request.GET.get("call_area", None):
                    qs2 = qs2.filter(is_internal_call=call_area)
                df_compare_employee = pd.DataFrame(qs2.annotate(day=F("created__date")).values("start", "end", "day"))
                if not df_compare_employee.empty:
                    dict_df["compare_employee"] = df_compare_employee

            for key, df in dict_df.items():
                df["duration"] = df["end"] - df["start"]
                df["minutes"] = df.duration.dt.seconds / 60
                df = df.pivot_table(index=["day"], values="minutes", aggfunc=["count", "sum", "mean", "median"])
                df["average_count_period"] = df["count"]["minutes"].mean()
                df["average_sum_period"] = df["sum"]["minutes"].mean()
                df["average_mean_period"] = df["mean"]["minutes"].mean()
                df["average_median_period"] = df["median"]["minutes"].mean()

                df[
                    [
                        "sum",
                        "mean",
                        "median",
                        "average_count_period",
                        "average_sum_period",
                        "average_mean_period",
                        "average_median_period",
                    ]
                ] = df[
                    [
                        "sum",
                        "mean",
                        "median",
                        "average_count_period",
                        "average_sum_period",
                        "average_mean_period",
                        "average_median_period",
                    ]
                ].round(2)
                df = df.reset_index()

                dict_df[key] = df

            df = dict_df["employee"]
            if compare_employee_id and not df_compare_employee.empty:
                df2 = dict_df["compare_employee"]

            if (business_day_without_call := self.request.GET.get("business_day_without_call", None)) and (
                business_day_without_call == "true"
            ):
                df = self.merge_df_with_business_days(df)
                if compare_employee_id and not df_compare_employee.empty:
                    df2 = self.merge_df_with_business_days(df2)

                # df = df.merge(df2[['day']], on=['day'], how = 'outer', indicator=True)
                # df[['count', 'sum', 'mean', 'median']] = df[['count', 'sum', 'mean', 'median']].fillna(0)
                # df[["average_count_period", "average_sum_period", "average_mean_period", "average_median_period"]] = df[["average_count_period", "average_sum_period", "average_mean_period", "average_median_period"]].fillna(0)
                # df = df.sort_values(by=['day']).reset_index()

                # df2 = df2.merge(df[['day']], on=['day'], how = 'outer', indicator=True)
                # df2[['count', 'sum', 'mean', 'median']] = df2[['count', 'sum', 'mean', 'median']].fillna(0)
                # df2[["average_count_period", "average_sum_period", "average_mean_period", "average_median_period"]] = df2[["average_count_period", "average_sum_period", "average_mean_period", "average_median_period"]].fillna(0)
                # df2 = df2.sort_values(by=['day']).reset_index()

            fig = make_subplots(
                rows=4, cols=1, shared_xaxes=True, vertical_spacing=0.01, row_heights=[0.2, 0.3, 0.25, 0.25]
            )

            fig.add_trace(
                go.Bar(
                    x=df["day"],
                    y=df["count"]["minutes"],
                    name="Number of calls",
                    text=df["count"]["minutes"],
                    textposition="auto",
                ),
                row=1,
                col=1,
            )
            if compare_employee_id and not df_compare_employee.empty:
                # df2 = dict_df["compare_employee"]
                fig.add_trace(
                    go.Bar(
                        x=df["day"],
                        y=df2["count"]["minutes"],
                        name="Number of calls " + compare_employee.computed_str,
                        text=df2["count"]["minutes"],
                        textposition="auto",
                        marker=dict(color="#A9A9A9"),
                    ),
                    row=1,
                    col=1,
                )

            fig.add_trace(
                go.Scatter(
                    x=df["day"],
                    y=df["average_count_period"],
                    mode="lines",
                    line=dict(dash="dash", width=2),
                    name="Average number of calls for the period",
                    opacity=0.5,
                    marker=dict(color="red"),
                ),
                row=1,
                col=1,
            )

            fig.add_trace(
                go.Bar(
                    x=df["day"],
                    y=df["sum"]["minutes"],
                    name="Total minutes",
                    text=df["sum"]["minutes"],
                    textposition="auto",
                    marker=dict(color="rgb(55, 83, 109)"),
                ),
                row=2,
                col=1,
            )
            if compare_employee_id and not df_compare_employee.empty:
                fig.add_trace(
                    go.Bar(
                        x=df2["day"],
                        y=df2["sum"]["minutes"],
                        name="Total minutes " + compare_employee.computed_str,
                        text=df2["sum"]["minutes"],
                        textposition="auto",
                        marker=dict(color="#696969"),
                    ),
                    row=2,
                    col=1,
                )

            fig.add_trace(
                go.Scatter(
                    x=df["day"],
                    y=df["average_sum_period"],
                    mode="lines",
                    line=dict(dash="dashdot", width=2),
                    name="Average Total minutes for the period",
                    opacity=0.5,
                    marker=dict(color="#FFD700"),
                ),
                row=2,
                col=1,
            )

            fig.add_trace(
                go.Bar(
                    x=df["day"],
                    y=df["median"]["minutes"],
                    name="Median minutes",
                    text=df["median"]["minutes"],
                    textposition="auto",
                    marker=dict(color="skyblue"),
                ),
                row=3,
                col=1,
            )
            if compare_employee_id and not df_compare_employee.empty:
                fig.add_trace(
                    go.Bar(
                        x=df2["day"],
                        y=df2["median"]["minutes"],
                        name="Median minutes " + compare_employee.computed_str,
                        text=df2["median"]["minutes"],
                        textposition="auto",
                        marker=dict(color="#D3D3D3"),
                    ),
                    row=3,
                    col=1,
                )

            fig.add_trace(
                go.Scatter(
                    x=df["day"],
                    y=df["average_median_period"],
                    mode="lines",
                    line=dict(dash="dash", width=2),
                    name="Median minutes for the period",
                    opacity=0.5,
                    marker=dict(color="orange"),
                ),
                row=3,
                col=1,
            )

            fig.add_trace(
                go.Bar(
                    x=df["day"],
                    y=df["mean"]["minutes"],
                    name="Average minutes",
                    text=df["mean"]["minutes"],
                    textposition="auto",
                    marker=dict(color="blue"),
                ),
                row=4,
                col=1,
            )
            if compare_employee_id and not df_compare_employee.empty:
                fig.add_trace(
                    go.Bar(
                        x=df2["day"],
                        y=df2["mean"]["minutes"],
                        name="Average minutes " + compare_employee.computed_str,
                        text=df2["mean"]["minutes"],
                        textposition="auto",
                        marker=dict(color="#808080"),
                    ),
                    row=4,
                    col=1,
                )

            fig.add_trace(
                go.Scatter(
                    x=df["day"],
                    y=df["average_mean_period"],
                    mode="lines",
                    line=dict(dash="dash", width=2),
                    name="Average minutes for the period",
                    opacity=0.5,
                    marker=dict(color="#BDB76B"),
                ),
                row=4,
                col=1,
            )

            # Update xaxis properties
            fig.update_xaxes(type="category", row=1, col=1, gridcolor="white")
            fig.update_xaxes(type="category", row=2, col=1, gridcolor="white")
            fig.update_xaxes(type="category", row=3, col=1, gridcolor="white")
            fig.update_xaxes(
                title_text="days", type="category", row=4, col=1, rangeslider_visible=True, gridcolor="white"
            )
            # Update yaxis properties
            fig.update_yaxes(title_text="Number of calls", title_font_size=12, row=1, col=1)
            fig.update_yaxes(title_text="Total minutes", title_font_size=12, row=2, col=1)
            fig.update_yaxes(title_text="Median Minutes", row=3, title_font_size=12, col=1)
            fig.update_yaxes(title_text="Average Minutes", row=4, title_font_size=12, col=1)

            fig.update_layout(
                title="<b>Call Summary Graph</b>",
                autosize=True,
                height=800,
                hovermode="x",
            )

        return fig


def callback_consent_permission(request):
    result = request.GET.get("admin_consent")
    return HttpResponse(result, content_type="text/plain")


# @require_POST
@csrf_exempt
def listen(request):
    # handle validation
    if request.GET.get("validationToken"):
        token = request.GET.get("validationToken")
        return HttpResponse(token, content_type="text/plain")

    # handle notifications
    # Latency for delivery of the change notification callRecord Less than 15 minutes Maximum latency	60 minutes
    notifications = None
    if request.body:
        # logger.warning(f"{timezone.now():%Y-%m-%d %H:%M:%S.%f} {request.body}")
        json_body = json.loads(request.body)
        if json_body.get("value"):
            notifications = parse(json_body.get("value"))
            for notification in notifications:
                # print(colored(f"{timezone.now():%Y-%m-%d %H:%M:%S.%f} {notification}", 'blue'))
                if (resource_data := notification.get("resource_data")) and (id_event := resource_data.get("id")):
                    transaction.on_commit(lambda: handle_event_from_webhook.delay(id_event, notification))
    return HttpResponse(notifications, content_type="text/plain")



import json
from datetime import timedelta

import pandas as pd
import requests
from django.conf import settings
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry
from rest_framework import status

from .parser import parse


class MicrosoftGraphAPI:
    def __init__(self):
        self.authority = getattr(settings, "WBINTEGRATOR_OFFICE365_AUTHORITY", "")
        self.client_id = getattr(settings, "WBINTEGRATOR_OFFICE365_CLIENT_ID", "")
        self.client_secret = getattr(settings, "WBINTEGRATOR_OFFICE365_CLIENT_SECRET", "")
        self.redirect_uri = getattr(settings, "WBINTEGRATOR_OFFICE365_REDIRECT_URI", "")
        self.token_endpoint = getattr(settings, "WBINTEGRATOR_OFFICE365_TOKEN_ENDPOINT", "")
        self.notification_url = getattr(settings, "WBINTEGRATOR_OFFICE365_NOTIFICATION_URL", "")
        self.graph_url = getattr(settings, "WBINTEGRATOR_OFFICE365_GRAPH_URL", "")

        global_preferences = global_preferences_registry.manager()
        if global_preferences["wbintegrator_office365__access_token"] == "0":
            global_preferences["wbintegrator_office365__access_token"] = self._get_access_token()

    def _get_administrator_consent(self):
        url = f"{self.authority}/adminconsent?client_id={self.client_id}&state=12345&redirect_uri={self.redirect_uri}"
        response = self._query(url, access_token=False)
        if response:
            return response
        else:
            raise ValueError("get administrator consent does not return response 200")

    def _get_access_token(self):
        # Get administrator consent
        self._get_administrator_consent()
        # Get an access token
        url = f"{self.authority}{self.token_endpoint}"
        payload = {
            "grant_type": "client_credentials",
            "client_id": self.client_id,
            "scope": "https://graph.microsoft.com/.default",
            "client_secret": self.client_secret,
        }

        response = self._query(url, method="POST", data=payload, is_json=False, access_token=False)
        data = None
        if response:
            if response.json():
                data = response.json().get("access_token")
        else:
            raise ValueError(response, response.json())
        return data

    def _subscribe(self, resource, minutes, change_type):
        data = {
            "changeType": change_type,
            "notificationUrl": self.notification_url,
            "resource": resource,
            "clientState": "secretClientValue",
            "latestSupportedTlsVersion": "v1_2",
        }
        if minutes:
            # maximum time of subscription 4230 minutes (under 3 days)
            date = timezone.now() + timedelta(minutes=minutes)
            date = date.strftime("%Y-%m-%dT%H:%M:%SZ")
            data["expirationDateTime"] = date

        url = f"{self.graph_url}/subscriptions"
        response = self._query(url, method="POST", data=json.dumps(data))
        data = None
        if response:
            if response.json():
                data = parse(response.json(), scalar_value=True)
                if data:
                    data = data[0]
        else:
            raise ValueError(response, response.json())
        return data

    def _unsubscribe(self, subscription_id):
        url = f"{self.graph_url}/subscriptions/{subscription_id}"
        return self._query(url, method="DELETE")

    def _renew_subscription(self, subscription_id, minutes=4230):
        url = f"{self.graph_url}/subscriptions/{subscription_id}"
        date = timezone.now() + timedelta(minutes=minutes)
        date = date.strftime("%Y-%m-%dT%H:%M:%SZ")
        data = {"expirationDateTime": date}
        response = self._query(url, method="PATCH", data=json.dumps(data))
        data = None
        if response:
            if response.json():
                data = parse(response.json(), scalar_value=True)
                if data:
                    data = data[0]
        else:
            raise ValueError(response, response.json())
        return data

    def subscriptions(self):
        url = f"{self.graph_url}/subscriptions"
        # curl -X GET -H "Authorization: Bearer eyJ0eXAiOiJKV1QiLCJub25jZSI6IlVUSmxpNTVOcDV1MHVDb3dwTWxpMThIdDdQRDZwV1VkMWNyNlRRYUNlNG8iLCJhbGciOiJSUzI1NiIsIng1dCI6Im5PbzNaRHJPRFhFSzFqS1doWHNsSFJfS1hFZyIsImtpZCI6Im5PbzNaRHJPRFhFSzFqS1doWHNsSFJfS1hFZyJ9.eyJhdWQiOiJodHRwczovL2dyYXBoLm1pY3Jvc29mdC5jb20iLCJpc3MiOiJodHRwczovL3N0cy53aW5kb3dzLm5ldC83ZWJhNTBhMi00N2QyLTQxODUtYWIwNy0yYTNjYTE4YzliNzYvIiwiaWF0IjoxNjE4MjE5ODIwLCJuYmYiOjE2MTgyMTk4MjAsImV4cCI6MTYxODIyMzcyMCwiYWlvIjoiRTJaZ1lBaVNGN3ZUS0tGbng4YTk2ak5mNjhFVEFBPT0iLCJhcHBfZGlzcGxheW5hbWUiOiJOb3RpZmljYXRpb25DYWxsVXNlckFwcGxpY2F0aW9uIiwiYXBwaWQiOiJkYjg5ZDYxNi0xYWJlLTQzNTgtYWRmNC0zYzE2ZWY3ZjQ0ZjAiLCJhcHBpZGFjciI6IjEiLCJpZHAiOiJodHRwczovL3N0cy53aW5kb3dzLm5ldC83ZWJhNTBhMi00N2QyLTQxODUtYWIwNy0yYTNjYTE4YzliNzYvIiwiaWR0eXAiOiJhcHAiLCJvaWQiOiIyNzNjODI1Yy00YTYyLTQyOWUtODdiZi0zOWYzZDY2ZTMwYWYiLCJyaCI6IjAuQVRvQW9sQzZmdEpIaFVHckJ5bzhvWXliZGhiV2lkdS1HbGhEcmZROEZ1OV9SUEE2QUFBLiIsInJvbGVzIjpbIkFjY2Vzc1Jldmlldy5SZWFkV3JpdGUuTWVtYmVyc2hpcCIsIk1haWwuUmVhZFdyaXRlIiwiVXNlci5SZWFkV3JpdGUuQWxsIiwiRGVsZWdhdGVkUGVybWlzc2lvbkdyYW50LlJlYWRXcml0ZS5BbGwiLCJDYWxlbmRhcnMuUmVhZCIsIk1haWwuUmVhZEJhc2ljLkFsbCIsIkdyb3VwLlJlYWQuQWxsIiwiQWNjZXNzUmV2aWV3LlJlYWRXcml0ZS5BbGwiLCJEaXJlY3RvcnkuUmVhZFdyaXRlLkFsbCIsIkNhbGxSZWNvcmRzLlJlYWQuQWxsIiwiSWRlbnRpdHlVc2VyRmxvdy5SZWFkLkFsbCIsIlVzZXIuSW52aXRlLkFsbCIsIkRpcmVjdG9yeS5SZWFkLkFsbCIsIlVzZXIuUmVhZC5BbGwiLCJVc2VyTm90aWZpY2F0aW9uLlJlYWRXcml0ZS5DcmVhdGVkQnlBcHAiLCJGaWxlcy5SZWFkLkFsbCIsIk1haWwuUmVhZCIsIkNoYXQuUmVhZC5BbGwiLCJVc2VyLkV4cG9ydC5BbGwiLCJJZGVudGl0eVByb3ZpZGVyLlJlYWQuQWxsIiwiQ2FsZW5kYXJzLlJlYWRXcml0ZSIsIklkZW50aXR5Umlza3lVc2VyLlJlYWQuQWxsIiwiQWNjZXNzUmV2aWV3LlJlYWQuQWxsIiwiTWFpbC5TZW5kIiwiVXNlci5NYW5hZ2VJZGVudGl0aWVzLkFsbCIsIkNvbnRhY3RzLlJlYWQiLCJJZGVudGl0eVJpc2tFdmVudC5SZWFkLkFsbCIsIk1haWwuUmVhZEJhc2ljIiwiQ2hhdC5SZWFkQmFzaWMuQWxsIiwiQ2FsbHMuQWNjZXNzTWVkaWEuQWxsIiwiQXBwbGljYXRpb24uUmVhZC5BbGwiLCJSZXBvcnRzLlJlYWQuQWxsIl0sInN1YiI6IjI3M2M4MjVjLTRhNjItNDI5ZS04N2JmLTM5ZjNkNjZlMzBhZiIsInRlbmFudF9yZWdpb25fc2NvcGUiOiJFVSIsInRpZCI6IjdlYmE1MGEyLTQ3ZDItNDE4NS1hYjA3LTJhM2NhMThjOWI3NiIsInV0aSI6Im9ONGJ1anMzaGtlOUVoUTRCTlV6QUEiLCJ2ZXIiOiIxLjAiLCJ4bXNfdGNkdCI6MTU1ODY4MjUxMX0.Jw5jVFFK2HIHXkWpR61eVGdVv-NnwsMBpHDRUur6UXrUlxtPwLAVvaklEILzALUNKAWe2Ic0k737tAq1C10DCkHF3iJ5bXQWTD0yAux1pZrHNYMMJ-bRvAdcuaLys5Amtas8gcYhj8mSpHdiSclW_17TxPDacSjvDwYEFOjgzAUYUtsoR5Q_q2H14QVR-PFd9WreqcOVy4ELNBBUhFdFjmV8Cb0lLUujW5Q0zBniOrwyRN4VYSdGEuyogoCOeICoNBdgwnLjXq6BMv6CLLSpzwEKDp-ikK7SETDd1uyDADYOUg-YuJ_D-ZIiobUNpA4UnBMNpfrUrCDzF84iox0vWA" 'https://graph.microsoft.com/v1.0/subscriptions'
        response = self._query(url)
        data = None
        if response:
            if datum := response.json():
                data = parse(datum.get("value"))
                url = datum.get("@odata.nextLink")
                while url:
                    response = self._query(url)
                    datum = response.json()
                    data += parse(datum.get("value"))
                    url = datum.get("@odata.nextLink")

        else:
            raise ValueError(response, response.json())
        return data

    def user(self, email):
        query_params = {"$select": "id, userPrincipalName, displayName"}
        url = f"{self.graph_url}/users/{email}"
        response = self._query(url, params=query_params)
        data = None
        if response:
            if response.json():
                data = parse(response.json(), scalar_value=True)
                if data:
                    data = data[0]
        else:
            raise ValueError(response, response.json())
        return data

    def users(self, filter_params=True):
        query_params = {
            "$select": "id,displayName,businessPhones,mobilePhone, userPrincipalName, mail,email, mailNickname, givenName, surname, imAddresses"
        }
        url = f"{self.graph_url}/users"
        if filter_params:
            response = self._query(url, params=query_params)
        else:
            response = self._query(url)
        data = None
        if response:
            data = parse(response.json().get("value"))
        else:
            raise ValueError(response, response.json())
        return data

    def call(self, call_id: str, raise_error: bool = False):
        url = f"{self.graph_url}/communications/callRecords/{call_id}"
        response = self._query(url)
        data = None
        if response and (json_data := response.json()):
            data = parse(pd.json_normalize(json_data))
            if data:
                data = data[0]
        elif raise_error:
            raise ValueError(response, response.json())
        return data

    # CREATION CALENDAR
    def create_calendar_group(self, user_id, data):
        url = f"{self.graph_url}/users/{user_id}/calendarGroups"
        return self._query_create(url, data)

    def create_calendar(self, user_id, data, id_calendar_group=None):
        if id_calendar_group:
            url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars"
        else:
            url = f"{self.graph_url}/users/{user_id}/calendars"
        return self._query_create(url, data)

    def create_calendar_event(self, user_id, data, id_calendar_group=None, id_calendar=None):
        if id_calendar:
            # A user's calendar in the default calendarGroup.
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events"
            if id_calendar_group:
                # A user's calendar in a specific calendarGroup.
                url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars/{id_calendar}/events"
        else:
            # A user's or group's default calendar.
            url = f"{self.graph_url}/users/{user_id}/events"
        return self._query_create(url, data)

    # LIST CALENDAR
    def get_list_calendar_groups(self, user_id):
        url = f"{self.graph_url}/users/{user_id}/calendarGroups"
        return self._query_get_list(url)

    def get_list_calendars(self, user_id, id_calendar_group=None):
        if id_calendar_group:
            url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars"
        else:
            url = f"{self.graph_url}/users/{user_id}/calendars"
        # Configure query parameters to modify the results
        # query_params = {
        #     "$select": "subject, organizer, createdDateTime, lastModifiedDateTime, start, end, reminderMinutesBeforeStart, isReminderOn, recurrence,  hasAttachments, importance, location, webLink, attendees",
        #     "$orderby": "createdDateTime DESC",
        # }
        return self._query_get_list(url)

    def get_list_calendar_events(self, user_id, id_calendar=None, id_calendar_group=None):
        if id_calendar:
            # A user's calendar in the default calendarGroup.
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events"
            if id_calendar_group:
                # A user's calendar in a specific calendarGroup.
                url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars/{id_calendar}/events"
        else:
            # A user's or group's default calendar.
            url = f"{self.graph_url}/users/{user_id}/events"
        return self._query_get_list(url)

    # GET CALENDAR
    def get_calendar_group(self, user_id, id_calendar_group=None):
        url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}"
        return self._query_get(url)

    def get_calendar(self, user_id, id_calendar, id_calendar_group=None):
        if id_calendar_group:
            url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars/{id_calendar}"
        else:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}"
        return self._query_get(url)

    def get_calendar_event(self, user_id, id_event, id_calendar=None, id_calendar_group=None):
        if id_calendar:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events/{id_event}"
            if id_calendar_group:
                url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}/calendars/{id_calendar}/events/{id_event}"
        else:
            url = f"{self.graph_url}/users/{user_id}/events/{id_event}"

        return self._query_get(url)

    # FORWARD CALENDAR EVENT
    def forward_calendar_event(self, user_id, id_event, data, id_calendar_group=None, id_calendar=None):
        if id_calendar:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events/{id_event}/forward"
            if id_calendar_group:
                url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}/calendars/{id_calendar}/events/{id_event}/forward"
        else:
            url = f"{self.graph_url}/users/{user_id}/events/{id_event}/forward"
        return self._query_create(url, data)

    def get_calendar_event_by_resource(self, resource):
        url = f"{self.graph_url}/{resource}"
        return self._query_get(url)

    def get_calendar_from_navigation_link(self, url):
        return self._query_get(url)

    # UPDATE CALENDAR
    def update_calendar_group(self, user_id, id_calendar_group, data):
        url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}"
        return self._query_update(url, data)

    def update_calendar(self, user_id, id_calendar, data, id_calendar_group=None):
        if id_calendar_group:
            url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars/{id_calendar}"
        else:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}"
        return self._query_update(url, data)

    def update_calendar_event(self, user_id, id_event, data, id_calendar=None, id_calendar_group=None):
        if id_calendar:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events/{id_event}"
            if id_calendar_group:
                url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}/calendars/{id_calendar}/events/{id_event}"
        else:
            url = f"{self.graph_url}/users/{user_id}/events/{id_event}"
        return self._query_update(url, data)

    # DELETE CALENDAR
    def delete_calendar_group(self, user_id, id_calendar_group):
        url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}"
        return self._query_delete(url)

    def delete_calendar(self, user_id, id_calendar, id_calendar_group=None):
        if id_calendar_group:
            url = f"{self.graph_url}/users/{user_id}/calendarGroups/{id_calendar_group}/calendars/{id_calendar}"
        else:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}"
        return self._query_delete(url)

    def delete_calendar_event(self, user_id, id_event, id_calendar=None, id_calendar_group=None):
        if id_calendar:
            url = f"{self.graph_url}/users/{user_id}/calendars/{id_calendar}/events/{id_event}"
            if id_calendar_group:
                url = f"{self.graph_url}/users/{user_id}/calendargroups/{id_calendar_group}/calendars/{id_calendar}/events/{id_event}"
        else:
            url = f"{self.graph_url}/users/{user_id}/events/{id_event}"
        return self._query_delete(url)

    def delete_calendar_event_by_resource(self, resource):
        url = f"{self.graph_url}/{resource}"
        return self._query_delete(url)

    def get_or_create_workbench_calendar(self, user_id, calendar_name):
        workbench_calendar = None
        datum = self.get_list_calendars(user_id=user_id)
        if calendar_name:
            calendars = {calendar.get("name"): calendar.get("id") for calendar in datum}
            if calendar_name in calendars.keys():
                workbench_calendar = self.get_calendar(user_id, calendars[calendar_name])
            else:
                data = {"name": calendar_name}
                workbench_calendar = self.create_calendar(user_id, data)
        else:
            for calendar in datum:
                if calendar.get("is_default_calendar") is True:
                    workbench_calendar = self.get_calendar(user_id, calendar.get("id"))
        return workbench_calendar

    def _query_get_list(self, url):
        response = self._query(url)
        datum = None
        if response:
            if response.json():
                if response.json().get("value"):
                    datum = parse(pd.json_normalize(response.json().get("value")))
        else:
            raise ValueError(response, response.json())
        return datum

    def _query_get(self, url, raise_error=False):
        response = self._query(url)
        data = None
        if response:
            if response.json():
                data = parse(pd.json_normalize(response.json()))
                if data:
                    data = data[0]
        elif raise_error:
            raise ValueError(response, response.json())
        return data

    def _query_create(self, url, data):
        response = self._query(url, method="POST", data=json.dumps(data))
        data = None
        if response.status_code < 400:
            try:
                if response.json():
                    data = parse(pd.json_normalize(response.json()))
                    if data:
                        data = data[0]
            except requests.exceptions.InvalidJSONError:
                pass  # print(response, response.__dict__)

        else:
            raise ValueError(response, response.__dict__)
        return data

    def _query_update(self, url, data):
        response = self._query(url, method="PATCH", data=json.dumps(data))
        data = None
        if response:
            if response.json():
                data = parse(pd.json_normalize(response.json()))
        else:
            if response.status_code not in [status.HTTP_503_SERVICE_UNAVAILABLE, status.HTTP_409_CONFLICT]:
                raise ValueError(response, response.json())
            else:
                import time

                time.sleep(60)
                if response := self._query(url, method="PATCH", data=json.dumps(data)):
                    if response.json():
                        data = parse(pd.json_normalize(response.json()))
        return data

    def _query_delete(self, url):
        response = self._query(url, method="DELETE")
        if response:
            return response
        else:
            raise ValueError(response)

    def _query(self, url, method="GET", data=None, params=None, access_token=True, is_json=True):
        headers = {"content-type": "application/json" if is_json else "application/x-www-form-urlencoded"}
        if access_token:
            global_preferences = global_preferences_registry.manager()
            headers["Authorization"] = f'Bearer {global_preferences["wbintegrator_office365__access_token"]}'
        if method == "POST":
            response = requests.post(url, data=data, headers=headers)
        elif method == "DELETE":
            response = requests.delete(url, headers=headers)
        elif method == "PATCH":
            response = requests.patch(url, data=data, headers=headers)
        else:
            response = requests.get(url, headers=headers, params=params)
        if response.status_code == status.HTTP_401_UNAUTHORIZED:
            new_token = self._get_access_token()
            if new_token != global_preferences["wbintegrator_office365__access_token"]:
                global_preferences["wbintegrator_office365__access_token"] = new_token
                return self._query(
                    url, method=method, data=data, params=params, access_token=access_token, is_json=is_json
                )
        else:
            return response



from django.contrib import admin
from django.shortcuts import redirect
from django.urls import path
from wbcore.admin import ExportCsvMixin, ImportCsvMixin

from wbintegrator_office365.models.event import fetch_tenantusers
from wbintegrator_office365.models.subscription import (
    resubscribe_as_task,
    verification_subscriptions,
)

from .models import CallEvent, CallUser, Event, EventLog, Subscription, TenantUser


@admin.register(TenantUser)
class TenantUserAdmin(admin.ModelAdmin, ImportCsvMixin, ExportCsvMixin):
    search_fields = ("tenant_id", "display_name", "mail", "phone", "profile__computed_str", "tenant_organization_id")
    list_display = (
        "tenant_id",
        "display_name",
        "mail",
        "phone",
        "profile",
        "tenant_organization_id",
    )
    change_list_template = "admin/tenant_change_list.html"
    list_filter = ("is_internal_organization",)
    readonly_fields = ("is_internal_organization",)

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [path("fetch_tenantusers/", self._fetch_tenantusers)]
        return my_urls + urls

    def _fetch_tenantusers(self, request):
        datum, count_added = fetch_tenantusers()
        self.message_user(
            request, str(len(datum)) + " tenants users found " + str(count_added) + " new tenants users added"
        )
        return redirect("..")


@admin.register(EventLog)
class EventLogAdmin(admin.ModelAdmin):
    search_fields = ("id", "change_type", "last_event__resource", "last_event__subscription_id", "id_event")
    list_display = (
        "id",
        "order_received",
        "change_type",
        "is_handled",
        "resource",
        "created",
        "changed",
        "last_event",
        "id_event",
    )
    ordering = ["-id", "-order_received"]
    list_filter = ("change_type", "created", "changed")


class EventLogTabularInline(admin.TabularInline):
    model = EventLog
    fields = ["id", "order_received", "change_type", "is_handled", "resource", "created", "changed"]
    readonly_fields = ("id", "order_received", "change_type", "resource", "created", "changed")
    extra = 0
    autocomplete_fields = ["last_event"]
    ordering = ["-id", "-order_received"]


@admin.register(Event)
class EventAdmin(admin.ModelAdmin):
    search_fields = (
        "id",
        "id_event",
        "resource",
        "subscription_id",
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
        "tenant_user__profile__computed_str",
        "tenant_user__tenant_organization_id",
    )
    list_display = (
        "id",
        "auto_inc_id",
        "nb_received",
        "type",
        "change_type",
        "tenant_user",
        "is_handled",
        "created",
        "changed",
        "uuid_event",
        "id_event",
        "subscription_id",
    )
    raw_id_fields = ["tenant_user"]
    ordering = ["-auto_inc_id"]
    inlines = [EventLogTabularInline]
    list_filter = ("type", "change_type", "created", "changed")


@admin.register(CallEvent)
class CallEventAdmin(admin.ModelAdmin):
    search_fields = (
        "id",
        "event__id",
        "organizer__tenant_user__display_name",
        "organizer__tenant_user__mail",
        "organizer__tenant_user__phone",
        "organizer__tenant_user__profile__computed_str",
        "organizer__tenant_user__tenant_organization_id",
    )
    list_display = (
        "id",
        "event",
        "organizer",
        "version",
        "start",
        "end",
        "created",
        "last_modified",
        "is_internal_call",
    )
    list_filter = ("is_internal_call", "created", "last_modified", "start", "end")


@admin.register(CallUser)
class CallUserAdmin(admin.ModelAdmin):
    search_fields = (
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
        "tenant_user__profile__computed_str",
        "tenant_user__tenant_organization_id",
    )
    list_display = ("id", "tenant_user", "is_guest", "is_phone")
    list_filter = ("is_guest", "is_phone")


@admin.register(Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    search_fields = (
        "id",
        "subscription_id",
        "resource",
        "tenant_user__display_name",
        "tenant_user__mail",
        "tenant_user__phone",
        "tenant_user__profile__computed_str",
        "tenant_user__tenant_organization_id",
    )
    list_display = (
        "id",
        "subscription_id",
        "expiration_date",
        "type_resource",
        "tenant_user",
        "is_enable",
        "created",
        "change_type",
        "resource",
        "notification_url",
    )
    list_filter = ("is_enable", "type_resource", "expiration_date", "created")

    def disable_selected_subscriptions(self, request, queryset):
        for subscription in queryset:
            subscription.is_enable = False
            subscription.save()

        self.message_user(
            request,
            "Operation completed, we unsubscribe the subscriptions selected",
        )

    def verification_selected_subscriptions(self, request, queryset):
        for subscription in queryset:
            verification_subscriptions.delay(subscription.id)
            self.message_user(
                request,
                "Verification completed for the subscriptions selected",
            )

    def renew_selected_subscriptions(self, request, queryset):
        for subscription in queryset:
            resubscribe_as_task.delay(subscription.id)
            self.message_user(request, "Active subscriptions are renewed on microsoft")
        return redirect("..")

    actions = [disable_selected_subscriptions, verification_selected_subscriptions, renew_selected_subscriptions]

    readonly_fields = [
        "subscription_id",
        "change_type",
        "expiration_date",
        "notification_url",
        "resource",
        "application_id",
        "creator_id",
        "client_state",
        "latest_supported_tls_version",
        "notification_content_type",
        "odata_context",
        "encryption_certificate_id",
        "encryption_certificate",
        "include_resource_data",
        "notification_query_options",
        "is_enable",
    ]
